# MicadoWISE Docker Setup

MicadoWISE can be ran through docker. Running through Docker is the easiest
way
* to ensure a consistent environment,
* to run MicadoWISE in a windows environment,
* to use functionality that requires a database.

Prebuild images are not yet available and have to be built manually.

## Initial Setup

First install and run [Docker desktop](https://www.docker.com/get-started).
Then create a clone of this repository, if not done so already, and enter the
directory of this README. Make sure to
[not mangle the line endings](https://stackoverflow.com/questions/10418975/how-to-change-line-ending-settings)
by setting `core.autocrlf` to `input`.

```shell script
git config --global core.autocrlf input
git clone https://gitlab.astro-wise.org/micado/micadowise.git
cd micadowise/docker
```

## Docker Build

The base docker image sets up everything needed to run MicadoWISE without a
database.

Building it requires you to create a file with the credentials to access the
OmegaCEN conda repository. Propagating this secret requires using buildkit.
Ask Gijs Verdoes Kleijn for the credentials.

### Windows

To build on Windows, where `condausername:condapassword` should be the conda
credentials:

```shell script
(echo |set /p=condausername:condapassword)>conda_download_key.txt
set DOCKER_BUILDKIT=1
docker build --secret id=conda_download_key,src="%CD%/conda_download_key.txt" -t micadoconda .
```

### Linux

To build on Linux, where `condausername:condapassword` should be the conda
credentials:

```shell script
echo "condausername:condapassword" > conda_download_key.txt
export DOCKER_BUILDKIT=1
docker build --secret id=conda_download_key,src="./conda_download_key.txt" -t micadoconda .
```

## Docker Compose

Docker compose sets up a database in order to use MicadoWISE with a database.

Start the database and MicadoWISE instance:

```shell script
docker-compose up
```

This will also start a Jupyter notebook and print out the URL that can be used
to access it.
Use the URL containing `localhost` or `127.0.0.1`.

The MicadoWISE root directory will be mounted in `/opt/project`, mimicking the
behaviour of PyCharm.

It is also possible to start a bash prompt in the instance (or run any other
command):

```shell script
docker exec -ti docker_micadowise_1 bash
```


### Running Tests

The easiest way to run all the tests is through Docker compose.
In the container, build the project, become database system user,
and then run the conda tests:

```shell script
cd /opt/project/
bash build.sh
source ~/bin/micado_system.sh
bash conda-recipe/run_test.sh
```

### Coverage

The tests are run with coverage, which can be imported into PyCharm:
```shell script
coverage xml
```

TODO: expand on this placeholder.

## Jupyter Notebook

The docker image can also be used standalone to run Jupyter notebook, that is
without using `docker-compose` (so no database).
First make sure that there is no other server running on port 8888 on the host,
that is, stop any existing Jupyter notebooks like the one started with
`docker-compose`.
Then, on the host, run
```shell script
docker run -ti -p 8888:8888 micadoconda:latest jupyter notebook --ip 0.0.0.0 --no-browser --allow-root
```

This will print out the URL that can be used to access the notebook.
Use the URL containing `localhost` or `127.0.0.1`.
