# -*- coding: utf-8 -*-
"""The BaseFrame is the base class for all image-based classes.

It represents a single-layer image.
"""

import tempfile

from astropy.io import fits

from micado.main.DataItem import DataItem


class BaseFrameExtra(DataItem):
    """BaseFrame."""

    # TODO: use these time formats again

    format_time_filename = "%Y%m%dT%H%M%SS%f"
    # Format for strftime/strptime for filenames

    format_time_fitsheader = "%Y-%m-%dT%H:%M:%S.%f"
    # Format for strftime/strptime for fits headers

    def save_fits(self, filename: str = None):
        """Save as FITS file."""
        # TODO: Ensure the path where FITS files are stored and read from
        # can be set.
        if not filename:
            if not self.filename:
                self.filename = tempfile.NamedTemporaryFile(
                    dir=".",
                    prefix="MCDWS_",
                    suffix=".fits",
                    delete=False,
                ).name
            filename = self.filename

        hduprimary = fits.PrimaryHDU(header=fits.Header(self.fits_keywords))

        hdusextension = []
        for exti, ext in enumerate(self.extensions, 1):
            if ext.image is not None:
                header = fits.Header(ext.fits_keywords)
                header["EXTNAME"] = f"DET{exti}.DATA"
                header["HDUCLAS2"] = "DATA"
                header["HDUCLAS3"] = ""
                hdu = fits.ImageHDU(
                    data=ext.image,
                    header=header,
                )
                hdusextension.append(hdu)
            if ext.uncertainty is not None:
                header = fits.Header()
                header["EXTNAME"] = f"DET{exti}.ERR"
                header["HDUCLAS2"] = "ERROR"
                header["HDUCLAS3"] = "RMSE"
                hdu = fits.ImageHDU(
                    data=ext.uncertainty,
                    header=header,
                )
                hdusextension.append(hdu)
            if ext.quality is not None:
                header = fits.Header()
                header["EXTNAME"] = f"DET{exti}.DQ"
                header["HDUCLAS2"] = "QUALITY"
                header["HDUCLAS3"] = "FLAG32BIT"
                hdu = fits.ImageHDU(
                    data=ext.quality,
                    header=header,
                )
                hdusextension.append(hdu)

        hdul = fits.HDUList([hduprimary] + hdusextension)
        hdul.writeto(filename, output_verify="fix", clobber=True)

        self.unique_filename()

    def unique_filename(self, input_filename=None, extra=None):
        """Generate unique filename."""
        extra = extra if extra is not None else []
        # TODO: The extra is currently ignored in order to put the filenames
        # in the FITS headers. The extra's should be reenabled or removed
        # entirely.
        DataItem.unique_filename(
            self,
            input_filename=input_filename,
            extra=[self.creation_date.strftime(self.format_time_filename)] + extra,
        )
