# -*- coding: utf-8 -*-
"""DataExtensionImage represents a single image, e.g. a FITS extension."""

import numpy

from micado.main.DataExtension import DataExtension
from micado.hardware.Detector import Detector


class DataExtensionImageExtra(DataExtension):
    """DataExtensionImage represents a single image, e.g. a FITS extension."""

    image: numpy.ndarray = None
    # The image data

    uncertainty: numpy.ndarray = None
    # The uncertainty in the image data

    quality: numpy.ndarray = None
    # The data quality flags

    def __init__(
        self,
        image=None,
        uncertainty=None,
        quality=None,
        header=None,
        detector: Detector = None,
        *args,
        **kwargs,
    ):
        """Initialize."""
        super().__init__(header=header, *args, **kwargs)
        self.image = image
        self.uncertainty = uncertainty
        self.quality = quality

        if detector is not None:
            if self.NAXIS1 <= 0:
                self.NAXIS1 = 2 * int(numpy.round(detector.xhw / detector.pixsize))
            if self.NAXIS2 <= 0:
                self.NAXIS2 = 2 * int(numpy.round(detector.yhw / detector.pixsize))
            if self.EXTNAME == "":
                # detector.id MUST be DET1 to DET9
                self.EXTNAME = f"{detector.id}.DATA".upper()

    def __eq__(self, other):
        """Test equality by comparing persistent properties and data."""
        if not super().__eq__(other):
            return False

        attrnames = ["image", "uncertainty", "quality"]
        for a in attrnames:
            v1 = getattr(self, a)
            v2 = getattr(other, a)
            if type(v1) != type(v2):
                return False
            if isinstance(v1, numpy.ndarray):
                if (v1 != v2).all():
                    return False
            elif v1 != v2:
                return False

        return True
