# -*- coding: utf-8 -*-
"""Tests for yaml_from_did.py."""
# TODO: Create more unittest-like tests.

import filecmp
import os
import sys
import tempfile
from pathlib import Path

import pytest
from micado.toolbox.yaml_from_did import main, PATH_DID


# TODO: Fix test on Windows
@pytest.mark.skipif(sys.platform.startswith("win"), reason="Test will fail on Windows")
def test_main():
    """Test the main function of yaml_from_did.py."""
    fns_base = [
        "ESO-ELT-DIC.MICADO_ATM",
        "ESO-ELT-DIC.MICADO_DET",
        "ESO-ELT-DIC.MICADO_INS",
        "ESO-ELT-DIC.MICADO_PRO",
        "ESO-ELT-DIC.MICADO_QC",
        "ESO-ELT-DIC.MICADO_SPEC_QC",
        "ESO-ELT-DIC.MICADO_REC",
        "ESO-ELT-DIC.MICADO_TEL",
        "ESO-ELT-DIC.MICADO_TPL",
        "ESO-VLT-DIC.PRIMARY",
        "ESO-VLT-DIC.PRO",
    ]
    fns_expected = [
        f"{fnb}.{fh}.{form}"
        for fnb in fns_base
        for fh in ["flat", "hierarch"]
        for form in ["json", "yaml"]
    ]
    path_yaml_expected = PATH_DID.parent / "yaml"
    assert os.path.exists(
        str(path_yaml_expected)
    ), f"ERROR expected {path_yaml_expected} does not exist"
    for fn in fns_expected:
        assert os.path.exists(str(path_yaml_expected / fn))

    with tempfile.TemporaryDirectory() as dir_yaml_temp:
        main(path_yaml=Path(dir_yaml_temp))
        fns = os.listdir(dir_yaml_temp)
        print(fns)
        assert set(fns) == set(fns_expected)
        fns_identical, fns_different, fns_error = filecmp.cmpfiles(
            str(path_yaml_expected),
            dir_yaml_temp,
            fns_expected,
            shallow=False,
        )
        assert set(fns_error) == set()
        assert set(fns_different) == set()
        assert set(fns_identical) == set(fns_expected)
        assert fns_different == []
        assert fns_error == []
