# -*- coding: utf-8 -8-
"""Tests for OCA creation."""
from pathlib import Path

import pytest

from micado.dataitems.Dark import Dark
from micado.dataitems.DpsicsStaroffsetSci import DpsicsStaroffsetSci
from micado.dataitems.Stack import Stack
from micado.dataitems.Staroffset import Staroffset
from micado.recipes.McdDetDark import McdDetDark
from micado.toolbox.oca_from_python import OCARules, OCAClassification, main

# Import many classes, because the class_cache is used to find derived classes.
# Therefore this test will behave differently when ran stand-alone or as part
# of the full test suite, unless all these classes are imported.
# noinspection PyUnresolvedReferences
import micado.main.aweimports  # noqa NOSONAR

from micado.main.commonwrapper import classcache

PATH_HERE = Path(__file__).resolve().parent


def test_oca_stack():
    """Test OCARules of MasterDark."""
    # This file is copied from the reflex directory.
    s_expected = open(PATH_HERE / "micado.oca").read()
    my_rules = OCARules()
    my_rules.add_dataitem(Stack)
    s_generated = str(my_rules)

    s_generated_clean = "\n".join(
        line
        for line in s_generated.strip().splitlines()
        if line and not line.startswith("//")
    )
    s_expected_clean = "\n".join(
        line
        for line in s_expected.strip().splitlines()
        if line and not line.startswith("//")
    )

    assert s_generated_clean == s_expected_clean


def test_oca_staroffset():
    """Test Staroffset."""
    # TODO: Placeholder.
    # These imports are necessary to properly test the handling of derived
    # classes. Without these imports the test will succeed when ran stand-alone
    # but will fail when running all functional tests together.
    # The print statements are just to placate SonarQube.
    print(Staroffset)
    print(DpsicsStaroffsetSci)


def test_oca_invalid():
    """Test creation of invalid rules."""
    with pytest.raises(ValueError):
        OCAClassification(McdDetDark)

    class DarkBad(Dark):
        """Class identical to Dark."""

    myrules = OCARules()
    with pytest.raises(ValueError):
        myrules.add_dataitem(Dark)
        myrules.add_dataitem(DarkBad)

    with pytest.raises(ValueError):
        # noinspection PyTypeChecker
        myrules.add_dataitem(McdDetDark)

    classcache.pop(DarkBad.__name__)


def test_oca_main():
    """Run the main OCA generation."""
    main()
